/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

// 27/03/12 20:20 NDP : Add ServerUserException that traps cleanly user exceptions

package org.awakefw.file.servlet;

import java.io.IOException;
import java.sql.SQLException;

import org.awakefw.commons.api.server.AwakeCommonsConfigurator;
import org.awakefw.file.api.server.AwakeFileConfigurator;
import org.awakefw.file.util.Tag;

/**
 * @author Nicolas de Pomereu
 *
 * Universal Exception message when a use class (configurator) throws an Exception
 */

public class ServerUserException {

    /**
     * Protected constructor
     */
    protected ServerUserException() {
		
    }

    /**
     * Throw a IOException that will be readable on client side and in logs
     * 
     * @param awakeCommonsConfigurator	the Awake Commons Configurator
     * @param e	the exception thrown 
     * @throws IOException
     */
    public static void throwUserIOException(
            AwakeCommonsConfigurator awakeCommonsConfigurator, Exception e,
            String method) throws IOException {
        String key = awakeCommonsConfigurator.getClass().getName();
        throw new IOException(
        	Tag.AWAKE_USER_CONFIG_FAIL
        		+ "The "
        		+ key
        		+ "."
        		+ method
        		+ "() method threw an Exception. Please correct the method on server.",
        	e);
    }
    
    /**
     * Throw a SQLException that will be readable on client side and in logs
     * 
     * @param awakeCommonsConfigurator	the Awake Commons Configurator
     * @param e	the exception thrown 
     * @throws IOException
     */
    public static void throwUserSQLException(
            AwakeCommonsConfigurator awakeCommonsConfigurator, Exception e,
            String method) throws SQLException {
        String key = awakeCommonsConfigurator.getClass().getName();
        throw new SQLException(
        	Tag.AWAKE_USER_CONFIG_FAIL
        		+ "The "
        		+ key
        		+ "."
        		+ method
        		+ "() method threw an Exception. Please correct the method on server.",
        	e);
    }
    
    /**
     * Throw a IOException that will be readable on client side and in logs
     * 
     * @param awakeFileConfigurator	the Awake File Configurator 
     * @param e	the exception thrown 
     * @throws IOException
     */
    public static void throwUserIOException(
            AwakeFileConfigurator awakeFileConfigurator, Exception e,
            String method) throws IOException {
        String key = awakeFileConfigurator.getClass().getName();
        throw new IOException(
        	Tag.AWAKE_USER_CONFIG_FAIL
        		+ "The "
        		+ key
        		+ "."
        		+ method
        		+ "() method threw an Exception. Please correct the method on server.",
        	e);
    }
    
}
